@@ -1,29 +1,23 @@
 package network
 
 import (
-	"bufio"
-	"bytes"
 	"context"
-	"fmt"
-	"io"
 	"sync"
 	"time"
 
 	"gitlab.com/gitlab-org/gitlab-runner/common"
-	"gitlab.com/gitlab-org/gitlab-runner/helpers"
+	"gitlab.com/gitlab-org/gitlab-runner/helpers/trace"
 )
 
 type clientJobTrace struct {
-	*io.PipeWriter
-
 	client         common.Network
 	config         common.RunnerConfig
 	jobCredentials *common.JobCredentials
 	id             int
-	bytesLimit     int
 	cancelFunc     context.CancelFunc
 
-	log           bytes.Buffer
+	buffer *trace.Buffer
+
 	lock          sync.RWMutex
 	state         common.JobState
 	failureReason common.JobFailureReason
@@ -62,6 +56,14 @@ func (c *clientJobTrace) Fail(err error, failureReason common.JobFailureReason)
 	c.finish()
 }
 
+func (c *clientJobTrace) Write(data []byte) (n int, err error) {
+	return c.buffer.Write(data)
+}
+
+func (c *clientJobTrace) SetMasked(masked []string) {
+	c.buffer.SetMasked(masked)
+}
+
 func (c *clientJobTrace) SetCancelFunc(cancelFunc context.CancelFunc) {
 	c.cancelFunc = cancelFunc
 }
@@ -83,17 +85,15 @@ func (c *clientJobTrace) setFailure(reason common.JobFailureReason) {
 }
 
 func (c *clientJobTrace) start() {
-	reader, writer := io.Pipe()
-	c.PipeWriter = writer
 	c.finished = make(chan bool)
 	c.state = common.Running
 	c.sentState = common.Running
-	go c.process(reader)
+	c.setupLogLimit()
 	go c.watch()
 }
 
 func (c *clientJobTrace) finish() {
-	c.Close()
+	c.buffer.Close()
 	c.finished <- true
 
 	// Do final upload of job trace
@@ -105,54 +105,13 @@ func (c *clientJobTrace) finish() {
 	}
 }
 
-func (c *clientJobTrace) writeRune(r rune) (n int, err error) {
-	c.lock.Lock()
-	defer c.lock.Unlock()
-
-	n, err = c.log.WriteRune(r)
-	if c.log.Len() < c.bytesLimit {
-		return
-	}
-
-	c.log.WriteString(c.limitExceededMessage())
-	err = io.EOF
-	return
-}
-
-func (c *clientJobTrace) process(pipe *io.PipeReader) {
-	defer pipe.Close()
-
-	stopped := false
-	reader := bufio.NewReader(pipe)
-
-	c.setupLogLimit()
-
-	for {
-		r, s, err := reader.ReadRune()
-		if s <= 0 {
-			break
-		} else if stopped {
-			// ignore symbols if job log exceeded limit
-			continue
-		} else if err == nil {
-			_, err = c.writeRune(r)
-			if err == io.EOF {
-				stopped = true
-			}
-		} else {
-			// ignore invalid characters
-			continue
-		}
-	}
-}
-
 func (c *clientJobTrace) incrementalUpdate() common.UpdateState {
 	c.lock.RLock()
 	state := c.state
-	trace := c.log
+	trace := c.buffer.Bytes()
 	c.lock.RUnlock()
 
-	if c.sentTrace != trace.Len() {
+	if c.sentTrace != len(trace) {
 		result := c.sendPatch(trace)
 		if result != common.UpdateSucceeded {
 			return result
@@ -171,7 +130,7 @@ func (c *clientJobTrace) incrementalUpdate() common.UpdateState {
 	return common.UpdateSucceeded
 }
 
-func (c *clientJobTrace) sendPatch(trace bytes.Buffer) common.UpdateState {
+func (c *clientJobTrace) sendPatch(trace []byte) common.UpdateState {
 	tracePatch, err := newTracePatch(trace, c.sentTrace)
 	if err != nil {
 		c.config.Log().Errorln("Error while creating a tracePatch", err.Error())
@@ -197,7 +156,7 @@ func (c *clientJobTrace) sendPatch(trace bytes.Buffer) common.UpdateState {
 func (c *clientJobTrace) resendPatch(id int, config common.RunnerConfig, jobCredentials *common.JobCredentials, tracePatch common.JobTracePatch) (update common.UpdateState) {
 	if !tracePatch.ValidateRange() {
 		config.Log().Warningln(id, "Full job update is needed")
-		fullTrace := c.log.String()
+		fullTrace := string(c.buffer.Bytes())
 
 		jobInfo := common.UpdateJobInfo{
 			ID:            c.id,
@@ -240,10 +199,10 @@ func (c *clientJobTrace) sendUpdate(state common.JobState) common.UpdateState {
 func (c *clientJobTrace) fullUpdate() common.UpdateState {
 	c.lock.RLock()
 	state := c.state
-	trace := c.log
+	trace := c.buffer.Bytes()
 	c.lock.RUnlock()
 
-	if c.sentTrace != trace.Len() {
+	if c.sentTrace != len(trace) {
 		c.sendPatch(trace) // we don't care about sendPatch() result, in the worst case we will re-send the trace
 	}
 
@@ -253,14 +212,14 @@ func (c *clientJobTrace) fullUpdate() common.UpdateState {
 		FailureReason: c.failureReason,
 	}
 
-	if c.sentTrace != trace.Len() {
-		traceString := trace.String()
+	if c.sentTrace != len(trace) {
+		traceString := string(trace)
 		jobInfo.Trace = &traceString
 	}
 
 	update := c.client.UpdateJob(c.config, c.jobCredentials, jobInfo)
 	if update == common.UpdateSucceeded {
-		c.sentTrace = trace.Len()
+		c.sentTrace = len(trace)
 		c.sentState = state
 		c.sentTime = time.Now()
 	}
@@ -295,22 +254,21 @@ func (c *clientJobTrace) watch() {
 }
 
 func (c *clientJobTrace) setupLogLimit() {
-	c.bytesLimit = c.config.OutputLimit
-	if c.bytesLimit == 0 {
-		c.bytesLimit = common.DefaultOutputLimit
+	bytesLimit := c.config.OutputLimit
+	if bytesLimit == 0 {
+		bytesLimit = common.DefaultOutputLimit
 	}
 	// configuration values are expressed in KB
-	c.bytesLimit *= 1024
-}
+	bytesLimit *= 1024
 
-func (c *clientJobTrace) limitExceededMessage() string {
-	return fmt.Sprintf("\n%sJob's log exceeded limit of %v bytes.%s\n", helpers.ANSI_BOLD_RED, c.bytesLimit, helpers.ANSI_RESET)
+	c.buffer.SetLimit(bytesLimit)
 }
 
 func newJobTrace(client common.Network, config common.RunnerConfig, jobCredentials *common.JobCredentials) *clientJobTrace {
 	return &clientJobTrace{
 		client:              client,
 		config:              config,
+		buffer:              trace.New(),
 		jobCredentials:      jobCredentials,
 		id:                  jobCredentials.ID,
 		updateInterval:      common.UpdateInterval,
